# 0.1.2a
##################################################################################
#   MIT License
#
#   Copyright (c) [2021] [René Horn]
#
#   Permission is hereby granted, free of charge, to any person obtaining a copy
#   of this software and associated documentation files (the "Software"), to deal
#   in the Software without restriction, including without limitation the rights
#   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#   copies of the Software, and to permit persons to whom the Software is
#   furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in all
#   copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#   SOFTWARE.
###################################################################################
import io
import os
import queue
import shutil
import threading
import urllib.parse
import urllib.parse
import urllib.request
from concurrent.futures import ThreadPoolExecutor
from time import sleep, strftime, time
from urllib.error import URLError, HTTPError

import urllib3
from eisenradio.api import ghettoApi

# logging.basicConfig(level=logging.DEBUG, format='[%(levelname)s] (%(threadName)-10s) %(message)s',)
# logging.basicConfig(level=logging.INFO, format='[%(levelname)s] (%(threadName)-10s) %(message)s',)

version = '1.4'
print(f'ghettorecorder {version} (eisen_radio module)')  # ########## version #############


class GBase:
    # class attribute
    dict_exit = {}  # all "single" started (this time four, each station)
    dict_error = {}

    sleeper = 2  # for exit of all threads
    pool = ThreadPoolExecutor(200)
    radio_base_dir = os.path.dirname(os.path.abspath(__file__)) + '//radiostations'  # if not set set_radio_base_dir()
    settings_path = os.path.dirname(os.path.abspath(__file__)) + '//settings.ini'  # if not set in set_settings_path()
    path = os.getcwd()
    path_to = path + '//'
    timer = 0

    def __init__(self, radio_base_dir=None, settings_path=None):
        self.instance_attr_time = 0
        self.trigger = False
        self.radio_base_dir = radio_base_dir
        self.settings_path = settings_path

    @staticmethod
    def make_directory(str_path):
        try:
            os.mkdir(str_path)
        except FileExistsError:
            pass
        else:
            print('\t--> created directory: ' + str_path)

    @staticmethod
    def remove_special_chars(str_name):
        ret_value = str_name.translate({ord(string): "" for string in '"!@#$%^*()[]{};:,./<>?\|`~=+"""'})
        return ret_value

    @staticmethod
    def this_time():
        time_val = strftime("%Y-%m-%d %H:%M:%S")
        return time_val

    def countdown(self, instance_attr_time):
        t = 0
        while not t == instance_attr_time:
            sleep(1)
            self.timer = t
            print(self.timer)
            t += 1
            if t == 0:
                self.trigger = True
        print(f' done {instance_attr_time} {self.trigger}')
        return self.trigger


class GIni(GBase):  # remnant of command line version

    ini_keys = {}  # cls attribute to store selections from ini file, works because of key[key] = value, else not
    srv_param_dict = {}  # all ini keys plus short url, suffix, server type stuff
    start_stop_recording = {}  # ini key: 'start' , 'stop'; while loop check start, working check stop go upper while
    # ini_key + '_single_title', ini_key + '_rec_from_here'
    cost_current_ini = ''  # ini key for cost_dict calc / should be a dict
    cost_dict = {}  # stores len of received headers to calc amount of data searching strings per day
    # list of search strings delimiter blank, first key is named 'STRINGS': Britney Phantom ไม่เคยจะจำ Elton Jim techno
    search_dict = {'STRINGS': 'Britney Spears ไม่เคยจะจำ Elton AC/DC techno Band feat. mix'}  # only show it is working
    list_items = []  # radio key names for display in terminal
    search_title_keys_list = []  # radio short keys, not start recording all streams, only searched titles
    content_type = {}  # header info audio/mpeg

    @staticmethod
    def parse_url_simple_url(radio_url):
        url = radio_url  # whole url is used for connection to radio server

        # 'http:' is first [0], 'ip:port/xxx/yyy' second item [1] in list_url_protocol
        list_url_protocol = url.split("//")
        list_url_ip_port = list_url_protocol[1].split("/")  # 'ip:port' is first item in list_url_ip_port
        radio_simple_url = list_url_protocol[0] + '//' + list_url_ip_port[0]
        return radio_simple_url


class GNet(GBase):
    http_pool = urllib3.PoolManager(num_pools=20)

    @staticmethod
    def load_url(url):
        # returns status code, if server is alive conn.getcode()
        # use urllib, urllib3 causes response to wait "forever" and timeout is not working either
        # print(f' load_url {url}')
        with urllib.request.urlopen(url, timeout=15) as response:
            return response.getcode()

    @staticmethod
    def is_server_alive(url, str_radio):
        # don't delete - urllib3 timeout=5, placebo, retries=None or =2, screw yourself, since half of conn. die
        # we have server up, but content not presented - zombie
        try:
            GNet.load_url(url)
        except HTTPError:
            print('HTTPError')
            GBase.dict_error[str_radio] = 'HTTPError - check radio homepage'
            return True
        except URLError as error:  # <urlopen error timed out>
            print(f' ---> {str_radio} server failed: {error} , {url}')
            GBase.dict_error[str_radio] = error
            print('URLError')
            return False
        return True

    @staticmethod
    def stream_filetype_url(url, str_radio):
        try:
            with urllib.request.urlopen(url, timeout=15) as response:
                headers = response.getheader('Content-Type')
        except Exception as ex:
            print(ex)
            return False

        content_type = ''
        if headers == 'audio/aacp' or headers == 'application/aacp':
            content_type = '.aacp'
        if headers == 'audio/aac':
            content_type = '.aac'
        if headers == 'audio/ogg' or headers == 'application/ogg':
            content_type = '.ogg'
        if headers == 'audio/mpeg':
            content_type = '.mp3'
        if headers == 'audio/x-mpegurl' or headers == 'text/html':
            content_type = '.m3u'
        # application/x-winamp-playlist , audio/scpls , audio/x-scpls ,  audio/x-mpegurl

        try:
            GIni.content_type[str_radio] = headers
            if len(headers) <= 5:
                GIni.content_type[str_radio] = 'audio/mpeg'
            # print(f' stream_filetype_url: {GIni.content_type[str_radio]}')
        except Exception as e:
            print(e)
            pass

        return content_type


class GRecorder:
    unknown_title_name = 'unknown_title'
    path_record_dict = {}  # {station : file path}
    current_song_dict = {}  # each thread writes the new title to the station key name {station : title}
    start_write_command = {}  # recorder head thread set command to copy first file
    search_pattern_found = {}

    record_active_dict = {}
    listen_active_dict = {}
    ghettoApi.init_ghetto_listen_active_dict(listen_active_dict)
    ghetto_measure = {}
    ghettoApi.init_ghetto_measurements(ghetto_measure)
    audio_stream_queue_dict = {}
    ghettoApi.init_ghetto_audio_stream(audio_stream_queue_dict)

    @staticmethod
    def ghetto_recorder_display_title(str_radio, str_action):
        """display on terminal"""
        # print(f' ghetto_recorder_display_title: {str_radio}')
        update_terminal = ''
        stream_song_name = ''
        stop_while = False
        try:
            stream_song_name = GRecorder.current_song_dict[str_radio]
        except KeyError:
            pass
        GRecorder.search_pattern_found[str_radio] = False

        while 1:

            if not len(stream_song_name) <= 2:
                #  GRecorder.current_song_dict[str_radio] = stream_song_name  # WRITE SONG NAME   # :DEACTIVATED:
                stream_song_name = GRecorder.current_song_dict[str_radio]  # :REVERSE:

                if not update_terminal == stream_song_name:
                    update_terminal = stream_song_name
                    try:
                        print(f'\t\t {GBase.this_time().split()[1]} title on {str_radio}: \t {update_terminal}')
                    except UnicodeEncodeError:
                        print('\t\t title on ' + str_radio + ':\t' + 'unicode error (check "$ localectl status")')
                        print('\t\t title on ' + str_radio + ':\t' + 'consider to add additional language support.')

                    except Exception as ex:
                        print('\t\t title on ' + str_radio + ':\t' + 'unknown error (print to your terminal)')
                        print(ex)
                    else:
                        pass
            # give recorder command to record, if we got a match from search gui
            #

            try:
                for _ in GIni.search_title_keys_list:
                    if _ == str_radio:
                        # search option was checked,
                        # call - def search_pattern_start_record, search string match?
                        got_it = GRecorder.search_pattern_start_record(stream_song_name, str_radio)
                        if got_it:
                            GIni.start_stop_recording[str_radio] = 'start'
                        else:
                            GIni.start_stop_recording[str_radio] = 'stop'
                            GRecorder.search_pattern_found[str_radio] = False

            except Exception as ex:
                print(ex)

            for sec in range(GBase.sleeper):
                if str_action == "listen":
                    if GBase.dict_exit[str_radio] or not GRecorder.listen_active_dict[str_radio]:
                        stop_while = True
                        break
                if str_action == "record":
                    if GBase.dict_exit[str_radio] or not GRecorder.record_active_dict[str_radio]:
                        stop_while = True
                        break
                sleep(1)
            if stop_while:
                break

    @staticmethod
    def search_pattern_start_record(title, str_radio):
        # search strings in title to see if we should start recording, test it in tk/tcl in ghettorecorder.win package
        search_list = []  # chop the string

        try:
            strings = GIni.search_dict[str_radio]  # dict with user search strings
            search_list = strings.encode('utf-8').lower().split(b' ')
        except KeyError:
            pass

        for search_str in search_list:

            if not len(search_str) <= 1:  # b'' match problem

                if not title.encode('utf-8').lower().find(search_str) == -1:  # find() returns -1, if not found
                    if not GRecorder.search_pattern_found[str_radio]:
                        print(f'<<<< match station: {str_radio} phrase: {search_str}')
                        GRecorder.search_pattern_found[str_radio] = True
                    return True
        return False

    @staticmethod
    def ghetto_recorder_head(directory_save, stream_suffix, str_radio, str_action):
        """create file name from current meta for recorder"""
        if str_action == "listen":
            return

        GRecorder.start_write_command[str_radio] = False
        sleep(3)  # chance to get stream meta info
        first_record = True

        display_info = ""
        while GRecorder.record_active_dict[str_radio] or not GBase.dict_exit[str_radio]:
            stream_info = GRecorder.current_song_dict[str_radio]  # unknown_title, None, icy metadata if avail

            if not display_info == stream_info:
                if not stream_info == "":
                    display_info = stream_info
                    stripped_info = GBase.remove_special_chars(stream_info)

                    if first_record:
                        fresh_file_path = directory_save + '//' + '_incomplete_' + stripped_info + stream_suffix
                        GRecorder.path_record_dict[str_radio] = fresh_file_path
                        GRecorder.start_write_command[str_radio] = True
                        first_record = False
                    else:
                        fresh_file_path = directory_save + '//' + stripped_info + stream_suffix
                        GRecorder.path_record_dict[str_radio] = fresh_file_path  # NEW PATH ... ... ...

            if not GRecorder.record_active_dict[str_radio] or GBase.dict_exit[str_radio]:
                stripped_info = GBase.remove_special_chars(stream_info)
                fresh_file_path = directory_save + '//' + '_incomplete_' + stripped_info + stream_suffix
                GRecorder.path_record_dict[str_radio] = fresh_file_path
                break
            sleep(1)

    @staticmethod
    def g_recorder_make_dir(path_to_save, str_radio):
        try:
            GBase.make_directory(path_to_save)
            return True
        except FileNotFoundError:
            GRecorder.current_song_dict[str_radio] = '[{-_-}] ZZZzz zz z... Recorder Failure!'
            GBase.dict_exit[str_radio] = True  # all str_radio treads-will-stop
            print('\t---> Recorder Write Failure in : ' + path_to_save)
            return False

    @staticmethod
    def g_recorder_await_head(str_radio):
        while not GBase.dict_exit[str_radio]:
            try:
                if GRecorder.start_write_command[str_radio]:
                    break
            except KeyError:
                pass
            else:
                sleep(.1)

    @staticmethod
    def g_recorder_path_transfer_test(str_radio):
        try:
            file_path = GRecorder.path_record_dict[str_radio]
            return file_path
        except KeyError:
            GRecorder.current_song_dict[str_radio] = '[{-_-}] ZZZzz zz z... Recorder Failure!'
            GBase.dict_exit[str_radio] = True  # all str_radio threads stop, major problem
            return False

    @staticmethod
    def g_recorder_request(url):
        # url was tested in Gnet
        try:
            request = GNet.http_pool.request('GET', url,
                                             headers={'Connection': 'keep-alive'},
                                             timeout=3000,
                                             preload_content=False)
            return request
        except Exception as e:
            print(e)
            return False

    @staticmethod
    def g_recorder_next_file(str_radio, full_file_path, record_file, ghetto_recorder, stream_request_size):
        # COMPLETE file path has changed, new file to write; got new metadata
        if not full_file_path == GRecorder.path_record_dict[str_radio]:
            try:
                if os.path.exists(full_file_path):
                    os.remove(full_file_path)
                record_file.flush()

                ghetto_size = os.path.getsize(ghetto_recorder)
                if int(ghetto_size) >= int(stream_request_size):
                    shutil.copyfile(ghetto_recorder, full_file_path)
            except Exception as ex:
                print(ex)

            record_file.seek(0)
            record_file.truncate()

        full_file_path = GRecorder.path_record_dict[str_radio]  # SET NEW PATH (title) after copy
        return full_file_path

    @staticmethod
    def g_recorder_teardown(str_radio, record_file, ghetto_recorder, stream_request_size):
        try:
            sleep(5)
            file_size = os.path.getsize(ghetto_recorder)
            if int(file_size) >= int(stream_request_size):
                awaited_head_incomplete_tag = GRecorder.path_record_dict[str_radio]
                shutil.copyfile(ghetto_recorder, awaited_head_incomplete_tag)

            record_file.seek(0)
            record_file.truncate()
            record_file.flush()
            record_file.close()

            if os.path.exists(ghetto_recorder):
                os.remove(ghetto_recorder)
        except OSError as ex:
            print(ex)

    @staticmethod
    def g_recorder_empty_queue(str_radio):
        while not GRecorder.audio_stream_queue_dict[str_radio + ',audio'].empty():
            GRecorder.audio_stream_queue_dict[str_radio + ',audio'].get()

    @staticmethod
    def g_recorder_write_queue(str_radio, chunk):
        if not GRecorder.audio_stream_queue_dict[str_radio + ',audio'].full():
            GRecorder.audio_stream_queue_dict[str_radio + ',audio'].put(chunk)

    @staticmethod
    def g_recorder_ask_bit_rate(url):
        try:
            with urllib.request.urlopen(url, timeout=15) as response:
                bit_rate = response.getheader('icy-br')
                return bit_rate
        except Exception as ex:
            print(ex)
            return False

    @staticmethod
    def g_recorder_calc_chunk_size(url):
        """avoid digital noise, delays and connection breaks"""
        stream_chunk_size = io.DEFAULT_BUFFER_SIZE * 4  # 8KB * x; HQ audio 320kB/s
        if GRecorder.g_recorder_ask_bit_rate(url):
            bit_rate = int(GRecorder.g_recorder_ask_bit_rate(url))
            if bit_rate <= 80:
                stream_chunk_size = io.DEFAULT_BUFFER_SIZE
            if (bit_rate > 80) and (bit_rate <= 160):
                stream_chunk_size = io.DEFAULT_BUFFER_SIZE * 2
            if (bit_rate > 160) and (bit_rate <= 240):
                stream_chunk_size = io.DEFAULT_BUFFER_SIZE * 3
        return stream_chunk_size

    @staticmethod
    def g_recorder_rec(request, str_radio, ghetto_recorder, stream_chunk_size, full_file_path):

        with open(ghetto_recorder, 'wb') as record_file:
            for chunk in request.stream(stream_chunk_size):
                record_file.write(chunk)
                new_file_path = GRecorder.g_recorder_next_file(str_radio,
                                                               full_file_path,
                                                               record_file,
                                                               ghetto_recorder,
                                                               stream_chunk_size)
                full_file_path = new_file_path

                if not GRecorder.record_active_dict[str_radio] or GBase.dict_exit[str_radio]:  # timer dict_exit
                    GRecorder.g_recorder_teardown(str_radio, record_file, ghetto_recorder, stream_chunk_size)
                    request.release_conn()
                    break

    @staticmethod
    def ghetto_recorder_tail(url, str_radio, path_to_save, suffix, str_action):

        audio_stream_queue = queue.Queue(maxsize=10)  # maxsize for safety, normal one in use
        stream_chunk_size = GRecorder.g_recorder_calc_chunk_size(url)

        GRecorder.audio_stream_queue_dict[str_radio + ',audio'] = audio_stream_queue
        ghetto_recorder = os.path.join(path_to_save, '__ghetto_recorder' + str(suffix))
        request = GRecorder.g_recorder_request(url)

        if str_action == "record":
            GRecorder.g_recorder_await_head(str_radio)
            full_file_path = GRecorder.g_recorder_path_transfer_test(str_radio)
            if not full_file_path:
                return
            if not GRecorder.g_recorder_make_dir(path_to_save, str_radio):
                return
            GRecorder.g_recorder_rec(request, str_radio, ghetto_recorder, stream_chunk_size, full_file_path)

        # k = 0
        if str_action == "listen":
            for chunk in request.stream(stream_chunk_size):
                # if k % 10 == 0:
                #     for thread in threading.enumerate():
                #         print(f' {thread.name}   {k}')
                # k += 1

                GRecorder.g_recorder_write_queue(str_radio, chunk)
                if not GRecorder.listen_active_dict[str_radio] or GBase.dict_exit[str_radio]:
                    GRecorder.g_recorder_empty_queue(str_radio)
                    request.release_conn()
                    break

    @staticmethod
    def metadata_header_info(request, str_radio, request_time):
        try:
            GRecorder.ghetto_measure[str_radio + ',request_time'] = request_time
            GRecorder.ghetto_measure[str_radio + ',suffix'] = request.headers['content-type']
        except KeyError:
            pass
        try:
            GRecorder.ghetto_measure[str_radio + ",icy_genre"] = request.headers["icy-genre"]
            GRecorder.ghetto_measure[str_radio + ",icy_name"] = request.headers["icy-name"]
            GRecorder.ghetto_measure[str_radio + ",icy_br"] = request.headers["icy-br"]
            GRecorder.ghetto_measure[str_radio + ",icy_url"] = request.headers["icy-url"]
        except KeyError:
            pass

    @staticmethod
    def metadata_icy_info(request):
        try:
            icy_meta_int = int(request.headers['icy-metaint'])
            request.read(icy_meta_int)
            start_byte = request.read(1)
            start_int = ord(start_byte)
            num_of_bytes = start_int * 16
            metadata_content = request.read(num_of_bytes)
            return metadata_content
        except Exception as e:
            print(e)
            return False

    @staticmethod
    def metadata_request(url):

        request = GNet.http_pool.request('GET', url,
                                         headers={'Icy-MetaData': '1'},
                                         preload_content=False)
        return request

    @staticmethod
    def metadata_get_display_extract(icy_info):
        # StreamTitle='X-Dream - Panic In Paradise * anima.sknt.ru';StreamUrl='';
        try:
            title_list = icy_info.split(";")
            if not len(title_list) > 1 or title_list is None:
                return  # empty list
            title_list = title_list[0].split("=")
            title = str(title_list[1])
            title = GBase.remove_special_chars(title)
            if title is not None:
                return title
        except IndexError:
            pass
        except OSError:
            pass
        return

    @staticmethod
    def metadata_get_display_info(icy_info):
        # <class 'bytes'> decode to <class 'str'> actually b''
        title = GRecorder.metadata_get_display_extract(icy_info.decode('utf-8'))
        if not title:
            return
        if title:
            return title

    @staticmethod
    def get_metadata_from_stream_loop(url, str_radio, str_action):
        stop_while = False
        icy_info = ''
        while 1:
            try:
                start_time = time()
                response = GRecorder.metadata_request(url)
                request_time = round((time() - start_time) * 1000)
                GRecorder.metadata_header_info(response, str_radio, request_time)
                icy_info = GRecorder.metadata_icy_info(response)
                response.release_conn()  # preload_content=False
            except HTTPError:
                pass
            except URLError:
                """url was checked in is_server_alive. assume short conn error"""
                pass

            title = GRecorder.metadata_get_display_info(icy_info)
            if title:
                try:
                    if title[0] == "'" and title[-1] == "'":
                        GRecorder.current_song_dict[str_radio] = title[1:-1]  # |sequence, ex 0 and last char |0 |1 |²2
                    else:
                        GRecorder.current_song_dict[str_radio] = title
                except KeyError:
                    pass
                except Exception as e:
                    print(e)
                    pass

            for sec in range(2):
                if str_action == "listen":
                    if GBase.dict_exit[str_radio] or not GRecorder.listen_active_dict[str_radio]:
                        stop_while = True
                        break
                if str_action == "record":
                    if GBase.dict_exit[str_radio] or not GRecorder.record_active_dict[str_radio]:
                        stop_while = True
                        break
                sleep(1)
            if stop_while:
                break

    @staticmethod
    def playlist_m3u(url, str_radio):
        # returns the first server of the playlist (not only m3u)
        try:
            read_url = GNet.http_pool.request('GET', url, preload_content=False)
        except Exception as ex:
            print(ex)
        else:
            file = read_url.read().decode('utf-8')

            m3u_lines = file.split("\n")
            # print(' \n    m3u_lines    ' + file)
            m3u_lines = list(filter(None, m3u_lines))  # remove empty rows
            m3u_streams = []
            for row_url in m3u_lines:
                if row_url[0:4].lower() == 'http'.lower():
                    m3u_streams.append(row_url)  # not to lower :)
                    # print(len(m3u_streams))

            if len(m3u_streams) > 1:
                print(f' {str_radio} Have more than one server in playlist_m3u. !!! Take first stream available.')
                play_server = m3u_streams[0]
                return play_server
            if len(m3u_streams) == 1:
                # print(' One server found in playlist_m3u')
                play_server = m3u_streams[0]
                return play_server
            if len(m3u_streams) == 0:
                # print(' No http ... server found in playlist_m3u !!! -EXIT-')
                return False


def check_alive_playlist_container(str_radio, str_url):
    # playlist url?
    if str_url[-4:] == '.m3u' or str_url[-4:] == '.pls':  # or url[-5:] == '.m3u8' or url[-5:] == '.xspf':
        # take first from the list
        is_playlist_server = GRecorder.playlist_m3u(str_url, str_radio)

        if not is_playlist_server == '':
            if GNet.is_server_alive(str_url, str_radio):
                str_url = is_playlist_server
            else:
                print('   --> playlist_server server failed, no recording')
            return str_url
    else:
        GNet.is_server_alive(str_url, str_radio)
        return False


def record_start_radio(str_radio, url, stream_suffix, dir_save, str_action):
    threading.Thread(name=str_radio + '_' + str_action + "_show", target=GRecorder.ghetto_recorder_display_title,
                     args=(str_radio, str_action),
                     daemon=True).start()
    threading.Thread(name=str_radio + '_' + str_action + "_head", target=GRecorder.ghetto_recorder_head,
                     args=(dir_save, stream_suffix, str_radio, str_action),
                     daemon=True).start()
    threading.Thread(name=str_radio + '_' + str_action + "_tail", target=GRecorder.ghetto_recorder_tail,
                     args=(url, str_radio, dir_save, stream_suffix, str_action),
                     daemon=True).start()
    threading.Thread(name=str_radio + '_' + str_action + "_meta", target=GRecorder.get_metadata_from_stream_loop,
                     args=(url, str_radio, str_action),
                     daemon=True).start()


def record(str_radio, url, str_action):
    stream_suffix = GNet.stream_filetype_url(url, str_radio)
    if str_action == "record":
        GRecorder.current_song_dict[str_radio] = GRecorder.unknown_title_name
    if str_action == "listen":
        GRecorder.current_song_dict[str_radio] = ""
    GIni.start_stop_recording[str_radio] = 'start'  # remnant of hit keyword, start record in tk/tcl g.recorder
    # GIni.start_stop_recording[str_radio] = 'stop'   # init it here, checkbox user interface
    GIni.start_stop_recording[str_radio + '_adv'] = 'start_from_here'

    dir_save = GBase.radio_base_dir + '//' + str_radio  # [Baustelle]
    record_start_radio(str_radio, url, stream_suffix, dir_save, str_action)

    # ################################## end ########################################
    # this version ends here. no loop



import copy
import concurrent.futures
from eisenradio.api import ghettoApi

""" read header info and request execution time """


rec_btn_archive = {}
lis_btn_archive = {}


def measure_meta():
    rv = cache_info()
    return rv


def stopped_stations():
    active_buttons_list = active_buttons()
    stop_list = streamer_stopped(active_buttons_list)
    renew_button_archive()
    return stop_list


def cache_info():
    streamer_list = active_buttons()
    rv = read_meta_from_api(streamer_list)
    return rv


def active_buttons():
    streamer_list = streamer_active_read()
    listener_str = listener_active_read()
    if listener_str:
        streamer_list.append(listener_str)
    return streamer_list


def streamer_active_read():
    active_streamer_list = []
    for radio_id, btn_down in ghettoApi.rec_btn_dict.items():
        if btn_down:
            active_streamer_list.append(radio_id)
    return active_streamer_list


def listener_active_read():
    for radio_id, btn_down in ghettoApi.lis_btn_dict.items():
        if btn_down:
            return radio_id
    return False


def streamer_stopped(active_streamer_list):
    global rec_btn_archive
    global lis_btn_archive

    stopp_list = []
    for radio_id, btn_down in rec_btn_archive.items():
        if btn_down:
            if radio_id not in active_streamer_list:
                stopp_list.append(radio_id)
    for radio_id, btn_down in lis_btn_archive.items():
        if btn_down:
            if radio_id not in active_streamer_list:
                if radio_id not in stopp_list:
                    stopp_list.append(radio_id)
    return stopp_list


def renew_button_archive():
    global rec_btn_archive
    global lis_btn_archive
    rec_btn_archive.clear()
    rec_btn_archive = copy.deepcopy(ghettoApi.rec_btn_dict)
    lis_btn_archive.clear()
    lis_btn_archive = copy.deepcopy(ghettoApi.lis_btn_dict)


def read_meta_from_api(streamer_list):
    name_list = radio_names(streamer_list)
    with concurrent.futures.ThreadPoolExecutor() as executor:
        rv = list(executor.map(meta, name_list))
    return rv


def radio_names(streamer_list):
    name_list = []
    for radio_id in streamer_list:
        name_list.append(ghettoApi.radios_in_view_dict[radio_id])
    return name_list


def meta(name):
    meta_info_list = [
        request_time_api(name),
        request_suffix_api(name),
        request_icy_genre_api(name),
        request_icy_name_api(name),
        request_icy_view_id_api(name),
        request_icy_br_api(name),
        request_icy_url_api(name)
    ]
    # list order:
    # request_time, request_suffix, request_icy_genre, request_icy_name,
    # request_icy_view_id, request_icy_br, request_icy_url
    return meta_info_list


def request_suffix_api(name):
    rv = '---'
    try:
        content_type = ghettoApi.ghetto_measure_dict[name + ',suffix']
        if content_type == 'audio/aacp' or content_type == 'application/aacp':
            rv = 'aacp'
        if content_type == 'audio/aac':
            rv = 'aac'
        if content_type == 'audio/ogg' or content_type == 'application/ogg':
            rv = 'ogg'
        if content_type == 'audio/mpeg':
            rv = 'mp3'
        if content_type == 'audio/x-mpegurl' or content_type == 'text/html':
            rv = 'm3u'
    except KeyError:
        pass
    return str(rv)


def request_icy_view_id_api(name):
    rv = '---'
    try:
        for key, val in ghettoApi.radios_in_view_dict.items():
            if val == name:
                rv = key
    except KeyError:
        pass
    return str(rv)


def request_time_api(name):
    rv = '---'
    try:
        rv = ghettoApi.ghetto_measure_dict[name + ',request_time']
    except KeyError:
        pass
    return str(rv)


def request_icy_genre_api(name):
    rv = '---'
    try:
        rv = ghettoApi.ghetto_measure_dict[name + ',icy_genre']
    except KeyError:
        pass
    return str(rv)


def request_icy_name_api(name):
    rv = '---'
    try:
        rv = ghettoApi.ghetto_measure_dict[name + ',icy_name']
    except KeyError:
        pass
    return str(rv)


def request_icy_br_api(name):
    rv = '---'
    try:
        rv = ghettoApi.ghetto_measure_dict[name + ',icy_br']
    except KeyError:
        pass
    return str(rv)


def request_icy_url_api(name):
    rv = '---'
    try:
        rv = ghettoApi.ghetto_measure_dict[name + ',icy_url']
    except KeyError:
        pass
    return str(rv)



# set GOOGLE_APPLICATION_CREDENTIALS=
# c:\Users\rene_\PycharmProjects\package_eisenradio_py\google-cloud\bigqueriy.access\eisenradio-bfb3346e65cd.json
# $env:GOOGLE_APPLICATION_CREDENTIALS=C:\Users\rene_\PycharmProjects\package_eisenradio_py\google-cloud\eisenradio-9a95136e40c9.json
from os import environ

environ['GOOGLE_APPLICATION_CREDENTIALS'] = 'C:\\Users\\rene_\\PycharmProjects\\package_eisenradio_py\\google-cloud' \
                                            '\\eisenradio-9a95136e40c9.json '

from google.cloud import bigquery

# Note: depending on where this code is being run, you may require
# additional authentication. See:
# https://cloud.google.com/bigquery/docs/authentication/
client = bigquery.Client()

query_job = client.query("""
SELECT COUNT(*) AS num_downloads
FROM `bigquery-public-data.pypi.file_downloads`
WHERE file.project = 'pytest'
  -- Only query the last 30 days of history
  AND DATE(timestamp)
    BETWEEN DATE_SUB(CURRENT_DATE(), INTERVAL 30 DAY)
    AND CURRENT_DATE()""")

results = query_job.result()  # Waits for job to complete.
for row in results:
    print("{} downloads".format(row.num_downloads))



import configparser
from os import path
from flask import flash
from eisenradio.lib.eisdb import delete_radio, get_db_connection, get_db_smaller, get_db_path, status_read_status_set
from eisenradio.eisenutil.eisutil import is_in_db_view
from eisenradio.lib.eisdb import render_picture


def delete_all_radios():
    rv = False
    posts = get_radios()

    for radio in posts:
        rv = delete_radio(radio['id'])
        if rv:
            rv = True
        if not rv:
            rv = False
    # vakuum db
    get_db_smaller()
    flash('Deletion Done', 'success')
    return rv


def export_radios():
    # BLUES_UK = http://149.255.59.3:8232/stream
    rv = True
    radio_url_dict = {}

    db_path = get_db_path()
    download_path = ''
    export_path = path.abspath(path.dirname(db_path))

    try:
        posts = get_radios()
        for station in posts:
            station_name = status_read_status_set(False, 'posts', 'title', station['id'])
            station_url = status_read_status_set(False, 'posts', 'content', station['id'])
            download_path = status_read_status_set(False, 'posts', 'download_path', station['id'])
            radio_url_dict[station_name] = station_url
    except KeyError:
        rv = False
    except ValueError:
        rv = False

    try:
        with open(path.join(export_path, 'settings.ini'), 'w') as writer:
            writer.write('[GLOBAL]' + '\n')
            writer.write('SAVE_TO_DIR = ' + download_path + '\n')
            writer.write('[STATIONS]' + '\n')
            for radio, url in radio_url_dict.items():
                writer.write(radio + ' = ' + url + '\n')
                writer.flush()
    except OSError:
        rv = False
    return rv


def import_radios(path_ini):
    ini_dict = find_all_in_stations(path_ini)

    if ini_dict:
        rv = db_import_ini(ini_dict, path_ini)
        if rv:
            return True
    return False


def find_all_in_stations(path_ini):
    config = configparser.ConfigParser()  # imported library to work with .ini files
    try:
        config.read_file(open(path_ini))
    except FileNotFoundError as ex:
        print(ex)
        return False
    else:
        station_dict = config['STATIONS']
        return station_dict


def find_save_to_dir(path_ini):
    config = configparser.ConfigParser()
    try:
        config.read_file(open(path_ini))
    except FileNotFoundError as ex:
        print(ex)
        return False
    else:
        global_dict = config['GLOBAL']
        return global_dict


def db_import_ini(ini_dict, path_ini):
    download_path = ''
    conn = get_db_connection()
    posts = conn.execute('SELECT * FROM posts').fetchall()

    for key, value in ini_dict.items():
        title = key
        content = value

        i = 1
        search_str = title
        while is_in_db_view(search_str):

            if not is_in_db_view(search_str + '__' + str(i)):
                title = search_str + '__' + str(i)
                break
            i += i

        radio_image, content_type = radio_spare_image()
        # if we later want extra save folders for each radio
        try:
            download_path = posts[0]["download_path"]
        except IndexError:
            global_dict = find_save_to_dir(path_ini)
            if global_dict:
                download_path = global_dict['SAVE_TO_DIR']
        except KeyError:
            print(' looks like the first radio to create, no save to path set')
            conn.execute('INSERT INTO posts (title, content, pic_data, pic_content_type) VALUES (?, ?, ?, ?)',
                         (title, content, radio_image, content_type))
        try:
            if download_path:
                conn.execute('INSERT INTO posts (title, content, download_path, pic_data, pic_content_type) VALUES ('
                             '?, ?, ?, ?, ?)',
                             (title, content, download_path, radio_image, content_type))
        except ValueError:
            conn.close()
            return False
    conn.commit()
    conn.close()
    return True


def get_radios():
    conn = get_db_connection()
    posts = conn.execute('SELECT * FROM posts').fetchall()
    conn.close()
    return posts


def radio_spare_image():
    this_dir = path.dirname(__file__)
    img_path = path.join(this_dir, 'bp_util_static', 'images', 'styles', 'drum_snail.png')
    with open(img_path, 'rb') as pic_reader:
        image_bin = pic_reader.read()
    img_base64 = render_picture(image_bin, 'encode')
    content_type = 'image/png'
    return img_base64, content_type





class Api:
    def __init__(self):
        self.config = None

    def init_app(self, app):
        self.config = app.config

    def __repr__(self):
        return f"api.init_app({self.config})"

    def __str__(self):
        return f"Flask application factory config: {self.config}"


class GhettoApi:

    def __init__(self):
        self.lis_btn_dict = None
        self.rec_btn_dict = None
        self.radios_in_view_dict = None
        self.ghetto_measure_dict = None
        self.ghetto_audio_stream_dict = None
        self.ghetto_show_analyser = True
        self.local_play_suffix = None
        self.local_play_queue = None
        self.listen_active_dict = None

    def init_lis_btn_dict(self, lis_btn_dict):
        self.lis_btn_dict = lis_btn_dict

    def init_rec_btn_dict(self, rec_btn_dict):
        self.rec_btn_dict = rec_btn_dict

    def init_radios_in_view(self, radios_in_view_dict):
        self.radios_in_view_dict = radios_in_view_dict

    """"show meta data"""
    def init_ghetto_measurements(self, ghetto_measure_dict):
        self.ghetto_measure_dict = ghetto_measure_dict

    """show frequency analyser, transfer buffer data for js AudioContext , ..dict[key + ',audio']"""
    def init_ghetto_audio_stream(self, ghetto_audio_stream_dict):
        self.ghetto_audio_stream_dict = ghetto_audio_stream_dict

    """set cookie to show or hide analyser, true or false"""
    def init_ghetto_show_analyser(self, ghetto_show_analyser):
        self.ghetto_show_analyser = ghetto_show_analyser

    """upload local sound to server to make byte array and get duration info (*aacp files), not working so far """
    def init_eisen_local_play_suffix(self, local_play_suffix):
        self.local_play_suffix = local_play_suffix

    """get sample rate, frequency and channel num to calculate time,some aac show no time, audio element stuck"""
    def init_eisen_local_play_queue(self, local_play_queue):
        self.local_play_queue = local_play_queue

    def init_ghetto_listen_active_dict(self, listen_active_dict):
        self.listen_active_dict = listen_active_dict


api = Api()
ghettoApi = GhettoApi()



from os import environ
from time import sleep
from eisenradio.api import ghettoApi


def stream_audio_feed(name):

    while True:

        try:
            if not ghettoApi.listen_active_dict[name]:
                return
            """grab from queue"""
            if not ghettoApi.ghetto_audio_stream_dict[name + ',audio'].empty():
                chunk = ghettoApi.ghetto_audio_stream_dict[name + ',audio'].get()
                yield chunk
        except KeyError:
            pass


def get_stream_content_type(name):
    content_type = ''
    while not content_type:

        try:
            content_type = ghettoApi.ghetto_measure_dict[name + ',suffix']
            """os variable, can resp. faster next time"""
            environ[name] = content_type
        except KeyError:
            pass

        sleep(.1)

    return content_type







import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="eisenradio", # project name /folder
    version="0.0.1",
    author="René Horn",
    author_email="rene_horn@gmx.net",
    description="Play radio. Record radio. Style your App.",
    long_description=long_description,
    license='gpl-3.0',
    long_description_content_type="text/markdown",
    url="",
    include_package_data=True,
    packages=setuptools.find_packages(),
        install_requires=[
		'flask_cors',
        'configparser',
        'requests',
        'urllib3'
    ],
    classifiers=[
    # How mature is this project? Common values are
    # https://packaging.python.org/guides/distributing-packages-using-setuptools/
    #   3 - Alpha
    #   4 - Beta
    #   5 - Production/Stable
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Topic :: Multimedia :: Sound/Audio :: Capture/Recording",
    ],
    python_requires='>=3.6',
)
# https://stackoverflow.com/questions/20288711/post-install-script-with-python-setuptools
# from setuptools import setup
def _post_install():
    print('----------------------------- POST INSTALL ----------------------------------')
    # setup(...)
    post_install()


from flask import Blueprint, render_template, request, flash, redirect, url_for, make_response, jsonify

import eisenradio.eisenhome.eishome as eisen_radio
from eisenradio.lib.eisdb import get_post, delete_radio
from eisenradio.api import ghettoApi

local_host_sound_route = "http://localhost:5050/sound/"


# Blueprint Configuration
eisenhome_bp = Blueprint(
    'eisenhome_bp', __name__,
    template_folder='bp_home_templates',
    static_folder='bp_home_static',
    static_url_path='/bp_home_static'
)


@eisenhome_bp.route('/', methods=('GET', 'POST'))
def index():
    global local_host_sound_route

    if eisen_radio.first_run_index:
        eisen_radio.check_write_protected()
    posts = eisen_radio.enum_radios()
    eisen_radio.display_clean_titles()
    eisen_radio.index_first_run(posts)
    current_station, current_table_id = eisen_radio.curr_radio_listen()

    if request.method == 'POST':
        # print_request_values(request.form.values())
        post_request = request.form.to_dict()  # flat dict werkzeug doc
        json_post = eisen_radio.index_posts_clicked(post_request)
        if json_post:
            return json_post

    listen_last_url = ""
    if current_station:
        listen_last_url = local_host_sound_route + current_station

    return render_template('bp_home_index.html',
                           posts=posts,
                           combo_master_timer=eisen_radio.combo_master_timer,
                           status_listen_btn_dict=eisen_radio.status_listen_btn_dict,
                           status_record_btn_dict=eisen_radio.status_record_btn_dict,
                           current_station=current_station,
                           current_table_id=current_table_id,
                           # listen_last_url=eisen_radio.listen_last_url    # may not be internet - cors zeroes out
                           listen_last_url=listen_last_url)


@eisenhome_bp.route('/page_flash', methods=('GET', 'POST'))
def page_flash():
    eisen_radio.combo_master_timer = 0  # master timer recording
    eisen_radio.progress_master_percent = 0

    flash('Count down timer ended all activities. App restart recommended!', 'success')
    return render_template('bp_home_page_flash.html')


@eisenhome_bp.route('/<int:id>/delete', methods=['POST'])
def delete(id):
    if eisen_radio.status_listen_btn_dict[id] or eisen_radio.status_record_btn_dict[id]:
        flash('Radio is active. No deletion.', 'warning')
        return redirect(url_for('eisenhome_bp.index'))

    post = get_post(id)
    rv = delete_radio(id)
    if rv:
        flash('"{}" was successfully deleted!'.format(post['title']), 'success')
    if not rv:
        flash('"{}" was not deleted!'.format(post['title']), 'warning')
    return redirect(url_for('eisenhome_bp.index'))


@eisenhome_bp.route('/setcookiedark', methods=['GET', 'POST'])
def setcookiedark():
    resp = make_response("Eisenkekse sind die besten")
    resp.set_cookie('eisen-cookie', 'darkmode', max_age=60*60*24*365*2, secure=False, httponly=True)
    return resp


@eisenhome_bp.route('/getcookiedark', methods=['GET'])
def getcookiedark():
    mode = request.cookies.get('eisen-cookie', None)
    return jsonify({'darkmode': mode})


@eisenhome_bp.route('/delcookiedark', methods=['POST'])
def delcookiedark():
    resp = make_response("necesito nuevas cookies")
    resp.set_cookie('eisen-cookie', max_age=0)
    return resp


"""cookie radio station listen"""


@eisenhome_bp.route('/cookie_set_station', methods=['GET', 'POST'])
def cookie_set_station():
    resp = make_response("current radio station plays\nleave index page and return, restore info")

    post_req = request.form.to_dict()  # flat dict werkzeug doc
    try:
        current_station = post_req['station']
        current_table_id = str(post_req['station_id'])
        str_station = current_station + ',' + str(current_table_id)
        resp.set_cookie('eisen-cookie-play-station', str_station, max_age=60 * 60 * 24 * 365 * 2, secure=False,
                        httponly=True)
    except KeyError:
        # no button click evt args in java
        current_station, current_table_id = eisen_radio.curr_radio_listen()
        str_station = current_station + ',' + str(current_table_id)
        if current_station == ',':
            resp.set_cookie('eisen-cookie-play-station', max_age=0)
            return resp
        resp.set_cookie('eisen-cookie-play-station', str_station, max_age=60 * 60 * 24 * 365 * 2, secure=False,
                        httponly=True)
    return resp


@eisenhome_bp.route('/cookie_get_station', methods=['GET'])
def cookie_get_station():
    play_station = request.cookies.get('eisen-cookie-play-station', None)
    return jsonify({'play_station': play_station})


@eisenhome_bp.route('/cookie_del_station', methods=['POST'])
def cookie_del_station():
    resp = make_response("radio stations are overrated\nbye")
    resp.set_cookie('eisen-cookie-play-station', max_age=0)
    return resp


@eisenhome_bp.route('/cookie_set_streamer', methods=['GET', 'POST'])
def cookie_set_streamer():
    """streamer to permanent cache"""
    resp = make_response("set streamer")
    str_streamer = ''

    for rec_station, rec_station_id in eisen_radio.active_streamer_dict.items():
        str_streamer = str_streamer + rec_station + '=' + rec_station_id + ','

    if not str_streamer:
        str_streamer = str('empty_json')
    resp.set_cookie('eisen-cookie-streamer', str_streamer, max_age=60*60*24*365*2, secure=False, httponly=True)
    return resp


@eisenhome_bp.route('/cookie_get_streamer', methods=['GET'])
def cookie_get_streamer():
    str_streamer = request.cookies.get('eisen-cookie-streamer', None)
    return jsonify({'str_streamer': str_streamer})


@eisenhome_bp.route('/cookie_del_streamer', methods=['POST'])
def cookie_del_streamer():
    resp = make_response("bye\nbye")
    resp.set_cookie('eisen-cookie-streamer', max_age=-1)
    return resp


@eisenhome_bp.route('/cookie_set_show_visuals', methods=['GET', 'POST'])
def cookie_set_show_visuals():
    resp = make_response("disable visualisation")
    resp.set_cookie('eisen-cookie-visuals', 'show_visuals', max_age=60*60*24*365*2, secure=False, httponly=True)
    ghettoApi.init_ghetto_show_analyser(False)
    return resp


@eisenhome_bp.route('/cookie_get_show_visuals', methods=['GET'])
def cookie_get_show_visuals():
    rv = request.cookies.get('eisen-cookie-visuals', None)

    if rv == 'show_visuals':
        return jsonify({'str_visuals': 'show_visuals'})
    if not rv:
        return jsonify({'str_visuals': '-empty-'})


@eisenhome_bp.route('/cookie_del_show_visuals', methods=['POST'])
def cookie_del_show_visuals():
    resp = make_response("bye\neisen-cookie-visuals")
    resp.set_cookie('eisen-cookie-visuals', max_age=-1)
    ghettoApi.init_ghetto_show_analyser(True)
    return resp


@eisenhome_bp.route('/index_posts_combo', methods=['POST'])
def index_posts_combo():
    eisen_radio.combo_master_timer = request.form['time_record_select_all']
    return eisen_radio.combo_master_timer


@eisenhome_bp.route('/index_posts_percent', methods=['POST'])
def index_posts_percent():
    return jsonify({'result': eisen_radio.progress_master_percent})


@eisenhome_bp.route('/display_info', methods=('GET', 'POST'))
def display_info():
    """updateDisplay() ajax"""
    if request.method == "GET":
        json_str = ''
        dict_display = {}
        for val in eisen_radio.active_displays:
            json_str = json_str + str(val) + '=' + str(eisen_radio.active_displays[val]) + ','
            dict_display[str(val)] = str(eisen_radio.active_displays[val])
        return jsonify({"result": json_str})     # dict_display


@eisenhome_bp.route('/is_play_station', methods=['GET'])
def is_play_station():
    current_station, current_table_id = eisen_radio.curr_radio_listen()
    if not current_station:
        current_station = '-empty-'
        current_table_id = '-empty-'

    return jsonify({'is_play_station_id': str(current_table_id), 'is_play_station_name': current_station})



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

# ###############################################

pack_name = "eisenradio"
pack_version = "1.0.4"
pack_description = "Play radio. Style your App."

INSTALL_REQUIRES = [
    'flask_cors',
    'configparser~=5.2.0',
    'requests~=2.26.0',
    'urllib3~=1.26.7',
    'waitress~=2.0.0',
    'certifi~=2021.10.8',
    'python-dotenv~=0.19.2',
    'setuptools~=59.2.0',
    'Flask~=2.0.2',
    'click~=8.0.3',
    'Werkzeug~=2.0.2',
    'pytest~=6.2.5',
]
PYTHON_REQUIRES = '>=3.6'

setuptools.setup(

    name=pack_name,  # project name /folder
    version=pack_version,
    author="René Horn",
    author_email="rene_horn@gmx.net",
    description=pack_description,
    long_description=long_description,
    license='MIT License',
    long_description_content_type="text/markdown",
    url="",
    include_package_data=True,
    packages=setuptools.find_packages(),
    install_requires=INSTALL_REQUIRES,
    classifiers=[
        # How mature is this project? Common values are
        # https://packaging.python.org/guides/distributing-packages-using-setuptools/
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "Topic :: Multimedia :: Sound/Audio :: Capture/Recording",
    ],
    python_requires=PYTHON_REQUIRES,
)



import sqlite3
import base64
import shutil
from os import path, makedirs, environ, remove
from pathlib import Path as Pathlib_path
from flask import flash, current_app
from werkzeug.exceptions import abort
from eisenradio.db import make_db_from_schema, empty_db_from_schema
from eisenradio.api import api


def get_db_path():

    try:
        db = environ['DATABASE']
        return db
    except KeyError:
        db = current_app.config["DATABASE"]
        return db


def get_db_connection():

    db = get_db_path()
    conn = sqlite3.connect(str(db))
    conn.row_factory = sqlite3.Row
    return conn


def status_read_status_set(set_status, table, column, table_id):
    # sqlite does not allow dynamic table or column assignment, user handbook
    col_value = ''

    conn = get_db_connection()

    if table == 'posts':
        """ tuple one field set(id,) """
        if column == 'content':
            col_value = conn.execute('SELECT content FROM posts WHERE id = ? ;', (table_id,)).fetchone()
        if column == 'title':
            col_value = conn.execute('SELECT title FROM posts WHERE id = ? ;', (table_id,)).fetchone()
        if column == 'download_path':
            col_value = conn.execute('SELECT download_path FROM posts WHERE id = ? ;', (table_id,)).fetchone()
        conn.close()
        if col_value:
            return col_value[0]
        if not col_value:
            return "column not in table posts, status_read_status_set"

    if table == 'eisen_intern':
        """ tuple two fields are set (id,str) """
        if column == 'browser_open':
            col_value = conn.execute(
                'SELECT browser_open FROM eisen_intern WHERE id = ?;', (str(table_id))).fetchone()

            if set_status:
                if col_value[0]:  # toggle values
                    conn.execute('UPDATE eisen_intern SET browser_open = ? WHERE id = ?;', (str(0), str(table_id)))
                else:
                    conn.execute('UPDATE eisen_intern SET browser_open = ? WHERE id = ?;', (str(1), str(table_id)))

        conn.commit()
        conn.close()

    return col_value[0]


def get_download_dir():
    dir_path = ''
    conn = get_db_connection()

    records = conn.execute('select id from posts').fetchall()
    if not records:
        print(f' No Radio in Database! Create a radio and set Save path in navigation bar')
        flash('No Radio in Database! Create a radio and set Save path in navigation bar', 'warning')
        return False
    for row_name_id in records:
        if row_name_id[0]:
            """take first item"""
            dir_path = conn.execute('SELECT download_path FROM posts WHERE id = ? ;', [str(row_name_id[0])]).fetchone()
            conn.close()
            break
        if not row_name_id[0]:
            dir_path = False
            break

    if dir_path:
        return dir_path[0]
    if not dir_path:
        print(f' No Radio in Database! Create a radio and set Save path in navigation bar')
        flash('No Radio in Database! Create a radio and set Save path in navigation bar', 'warning')
    return False


def get_db_smaller():
    db = get_db_path()
    conn = sqlite3.connect(db, isolation_level=None)
    conn.execute("VACUUM")
    conn.close()


def render_picture(byte_data, de_enc):
    render_pic = ''
    if de_enc == 'encode':
        render_pic = base64.b64encode(byte_data).decode('ascii')
    if de_enc == 'decode':
        render_pic = base64.b64decode(byte_data).decode('ascii')
    return render_pic


def get_post(post_id):
    conn = get_db_connection()
    post = conn.execute('SELECT * FROM posts WHERE id = ?',
                        [str(post_id)]).fetchone()
    conn.close()
    if post is None:
        abort(404)  # flask.abort(404)
    return post


def install_new_db(db_path):

    schema_db = ''

    is_snap_device = 'SNAP' in environ
    if not is_snap_device:
        # get root of app, rip off one subdir from script dir .go up
        schema_dir = path.dirname(path.dirname(__file__))
        schema_db = path.join(schema_dir, 'database.db')
    if is_snap_device:
        schema_dir = environ["SNAP_USER_COMMON"]
        schema_db = path.join(schema_dir, 'pre_configured.db')

    db_env = path.abspath(environ['DATABASE'])
    db_flask = path.abspath(api.config.get("DATABASE"))

    if str(db_env) != str(db_flask):
        print(f' -ValueError->{db_env}<-ValueError- environ')
        print(f' -ValueError->{db_flask}<-ValueError- api')
        raise ValueError("\nDatabase path do not match in os env and flask env!\nMost likely no clean up in Test; "
                         "RESTART!\nrh")

    """ if not path exists """
    if not path.exists(db_path):
        try:
            makedirs(path.dirname(db_path))
        except FileExistsError:
            pass

    this_db = Pathlib_path(db_path)
    testing = environ["TESTING"]
    if testing == 'False':
        if not this_db.is_file():
            make_db_from_schema()

    if testing == 'True':
        if this_db.is_file():
            remove(this_db)

        empty_db = environ["EMPTY_DB"]
        print(f' environ["EMPTY_DB"] for Test ->{environ["EMPTY_DB"]}<- in eisdb.py')

        if empty_db == 'True':
            empty_db_from_schema()
        if empty_db == 'False':
            make_db_from_schema()
        shutil.move(str(schema_db), db_path)

    if not this_db.is_file():
        shutil.move(str(schema_db), db_path)


def delete_radio(radio_id):
    conn = get_db_connection()
    try:
        conn.execute('DELETE FROM posts WHERE id = ?', (str(radio_id),))
        conn.commit()
        conn.close()
        get_db_smaller()
    except KeyError:
        conn.close()
        return False
    conn.close()
    return True
